@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,10 +34,10 @@ namespace xmltooling {
     class XMLTOOL_API ElementProxy : public virtual AttributeExtensibleXMLObject, public virtual ElementExtensibleXMLObject
     {
     protected:
-        ElementProxy() {}
+        ElementProxy();
 
     public:
-        virtual ~ElementProxy() {}
+        virtual ~ElementProxy();
     };
     
 };
