@@ -20,3 +20,14 @@
 
 /* Define if you wish to disable Xalan-dependent features. */
 #define XSEC_NO_XALAN
+
+#include <xercesc/util/XercesVersion.hpp>
+
+#if (XERCES_VERSION_MAJOR >= 3)
+# define XMLTOOLING_XERCESC_COMPLIANT_DOMLS     1
+# define XMLTOOLING_XERCESC_BOOLSETIDATTRIBUTE  1
+# define XMLTOOLING_XERCESC_64BITSAFE           1
+#endif
+
+/* Define to 1 if you have the `xsecsize_t' type. */
+#define HAVE_XSECSIZE_T 1
