@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 
 using namespace xmltooling;
+using namespace xercesc;
 using namespace std;
 using xmlconstants::XMLTOOLING_NS;
 
@@ -129,13 +130,13 @@ inline const char* get_digit_character()
     return s_mid;
 }
 
-inline const char* unsigned_integer_to_string(char* buf, size_t cchBuf, int i)
+inline const char* unsigned_integer_to_string(char* buf, size_t cchBuf, size_t i)
 {
     char* psz=buf + cchBuf - 1;     // Set psz to last char
     *psz = 0;                       // Set terminating null
 
     do {
-        unsigned int lsd = i % 10;  // Get least significant
+        size_t lsd = i % 10;  // Get least significant
                                     // digit
 
         i /= 10;                    // Prepare for next most
@@ -153,7 +154,7 @@ inline const char* unsigned_integer_to_string(char* buf, size_t cchBuf, int i)
 void XMLToolingException::addProperties(const params& p)
 {
     m_processedmsg.erase();
-    int i=m_params.size()+1;
+    map<string,string>::size_type i=m_params.size()+1;
     char buf[20];
     const vector<const char*>& v=p.get();
     for (vector<const char*>::const_iterator ci=v.begin(); ci!=v.end(); ci++) {
@@ -248,9 +249,10 @@ string XMLToolingException::toString() const
     const char* msg=getMessage();
     if (msg)
         xml_encode(xml, "<message>", msg, "</message>");
+    const URLEncoder* encoder = XMLToolingConfig::getConfig().getURLEncoder();
     for (map<string,string>::const_iterator i=m_params.begin(); i!=m_params.end(); i++) {
         xml_encode(xml, "<param name='", i->first.c_str(), "'");
-        xml_encode(xml, ">", i->second.c_str(), "</param>");
+        xml_encode(xml, ">", encoder->encode(i->second.c_str()).c_str(), "</param>");
     }
     xml+="</exception>";
     return xml;
@@ -294,13 +296,16 @@ XMLToolingException* XMLToolingException::fromStream(std::istream& in)
         excep->setMessage(m.get());
     }
     
+    const URLEncoder* encoder = XMLToolingConfig::getConfig().getURLEncoder();
     child=XMLHelper::getFirstChildElement(root,XMLTOOLING_NS,param);
     while (child && child->hasChildNodes()) {
         auto_ptr_char n(child->getAttributeNS(NULL,name));
-        char* v=toUTF8(child->getFirstChild()->getNodeValue());
-        if (n.get() && v)
-            excep->addProperty(n.get(), v);
-        delete[] v;
+        char* encoded = XMLString::transcode(child->getFirstChild()->getNodeValue());
+        if (n.get() && encoded) {
+            encoder->decode(encoded);
+            excep->addProperty(n.get(), encoded);
+        }
+        XMLString::release(&encoded);
         child=XMLHelper::getNextSiblingElement(child,XMLTOOLING_NS,param);
     }
 
