@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,22 +24,23 @@
 #define __xmltooling_x509cred_h__
 
 #include <xmltooling/security/Credential.h>
-#include <xmltooling/security/XSECCryptoX509CRL.h>
 
-#include <xsec/enc/XSECCryptoX509.hpp>
+class XSECCryptoX509;
 
 namespace xmltooling {
 
+    class XMLTOOL_API XSECCryptoX509CRL;
+
     /**
      * Wraps an X.509-based Credential.
      */
     class XMLTOOL_API X509Credential : public virtual Credential
     {
     protected:
-        X509Credential() {}
+        X509Credential();
         
     public:
-        virtual ~X509Credential() {}
+        virtual ~X509Credential();
 
         /**
          * Bitmask constants for limiting resolution process inside a CredentialResolver. 
@@ -67,12 +68,22 @@ namespace xmltooling {
         virtual const std::vector<XSECCryptoX509*>& getEntityCertificateChain() const=0;
 
         /**
+         * @deprecated
+         *
          * Gets a CRL associated with the credential.
          * 
          * @return CRL associated with the credential
          */
         virtual XSECCryptoX509CRL* getCRL() const=0;
 
+        /**
+         * Gets an immutable collection of all CRLs associated with the credential.
+         * 
+         * @return CRLs associated with the credential
+         */
+        virtual const std::vector<XSECCryptoX509CRL*>& getCRLs() const=0;
+
+
         /**
          * Gets the subject name of the first certificate in the chain.
          *
