@@ -27,12 +27,15 @@
 #include "util/NDC.h"
 #include "util/XMLHelper.h"
 
+#include <algorithm>
 #include <xercesc/util/XMLUniDefs.hpp>
 
 using namespace xmltooling::logging;
 using namespace xmltooling;
 using namespace std;
 
+using xercesc::DOMElement;
+
 namespace xmltooling {
     class XMLTOOL_DLLLOCAL ChainingCredentialResolver : public CredentialResolver
     {
